//
//  config.h
//  RSvIDX_V4
//
//  Created by Ruben Ticehurst-James on 03/01/2023.
//

#ifndef config_h
#define config_h


#define DATA_TYPE float
#define ROW_COLUMN_TYPE size_t
#define HASH_SIZE uint64_t
#define ID_SIZE 32

#endif /* config_h */
